@@ -82,13 +82,46 @@
  *             - Allow full filter specifications in -f
  */
 
+#define LDAP_DEPRECATED 1
+
+#include "util.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
+
+#ifdef _SQUID_MSWIN_		/* Native Windows port and MinGW */
+
+#define snprintf _snprintf
+#include <windows.h>
+#include <winldap.h>
+#ifndef LDAPAPI
+#define LDAPAPI __cdecl
+#endif
+#ifdef LDAP_VERSION3
+#define LDAP_OPT_SUCCESS LDAP_SUCCESS
+/* Some tricks to allow dynamic bind with ldap_start_tls_s entry point at
+ * run time.
+ */
+#undef ldap_start_tls_s
+#if LDAP_UNICODE
+#define LDAP_START_TLS_S "ldap_start_tls_sW"
+typedef WINLDAPAPI ULONG(LDAPAPI * PFldap_start_tls_s) (IN PLDAP, OUT PULONG, OUT LDAPMessage **, IN PLDAPControlW *, IN PLDAPControlW *);
+#else
+#define LDAP_START_TLS_S "ldap_start_tls_sA"
+typedef WINLDAPAPI ULONG(LDAPAPI * PFldap_start_tls_s) (IN PLDAP, OUT PULONG, OUT LDAPMessage **, IN PLDAPControlA *, IN PLDAPControlA *);
+#endif /* LDAP_UNICODE */
+PFldap_start_tls_s Win32_ldap_start_tls_s;
+#define ldap_start_tls_s(l,s,c) Win32_ldap_start_tls_s(l,NULL,NULL,s,c)
+#endif /* LDAP_VERSION3 */
+
+#else
+
 #include <lber.h>
 #include <ldap.h>
-#include <ctype.h>
-#include "util.h"
+
+#endif
 
 #define PROGRAM_NAME "squid_ldap_auth"
 
@@ -187,7 +220,7 @@ squid_ldap_set_referrals(LDAP * ld, int referrals)
     else
 	ld->ld_options &= ~LDAP_OPT_REFERRALS;
 }
-static void 
+static void
 squid_ldap_set_timelimit(LDAP * ld, int timelimit)
 {
     ld->ld_timelimit = timelimit;
@@ -259,7 +292,7 @@ open_ldap_connection(const char *ldapServer, int port)
     }
     if (use_tls) {
 #ifdef LDAP_OPT_X_TLS
-        if (version != LDAP_VERSION3) {
+	if (version != LDAP_VERSION3) {
 	    fprintf(stderr, "TLS requires LDAP version 3\n");
 	    exit(1);
 	} else if (ldap_start_tls_s(ld, NULL, NULL) != LDAP_SUCCESS) {
@@ -282,12 +315,12 @@ open_ldap_connection(const char *ldapServer, int port)
 static int
 validUsername(const char *user)
 {
-    const unsigned char *p = (const unsigned char *)user;
+    const unsigned char *p = (const unsigned char *) user;
 
     /* Leading whitespace? */
     if (isspace(p[0]))
 	return 0;
-    while(p[0] && p[1]) {
+    while (p[0] && p[1]) {
 	if (isspace(p[0])) {
 	    /* More than one consequitive space? */
 	    if (isspace(p[1]))
@@ -514,6 +547,22 @@ main(int argc, char **argv)
 	fprintf(stderr, "\tIf you need to bind as a user to perform searches then use the\n\t-D binddn -w bindpasswd or -D binddn -W secretfile options\n\n");
 	exit(1);
     }
+/* On windows ldap_start_tls_s is available starting from Windows XP, 
+ * so we need to bind at run-time with the function entry point
+ */
+#ifdef _SQUID_MSWIN_
+    if (use_tls) {
+
+	HMODULE WLDAP32Handle;
+
+	WLDAP32Handle = GetModuleHandle("wldap32");
+	if ((Win32_ldap_start_tls_s = (PFldap_start_tls_s) GetProcAddress(WLDAP32Handle, LDAP_START_TLS_S)) == NULL) {
+	    fprintf(stderr, PROGRAM_NAME ": ERROR: TLS (-Z) not supported on this platform.\n");
+	    exit(1);
+	}
+    }
+#endif
+
     while (fgets(buf, 256, stdin) != NULL) {
 	user = strtok(buf, " \r\n");
 	passwd = strtok(NULL, "\r\n");
@@ -581,17 +630,23 @@ ldap_escape_value(char *escaped, int size, const char *src)
     return n;
 }
 
+/* Check the userid & password.
+ * Return 0 on success, 1 on failure
+ */
 static int
 checkLDAP(LDAP * persistent_ld, const char *userid, const char *password, const char *ldapServer, int port)
 {
     char dn[256];
     int ret = 0;
     LDAP *bind_ld = NULL;
+    int rc;
 
     if (!*password) {
 	/* LDAP can't bind with a blank password. Seen as "anonymous"
 	 * and always granted access
 	 */
+	if (debug)
+	    fprintf(stderr, "Blank password given\n");
 	return 1;
     }
     if (searchfilter) {
@@ -599,9 +654,9 @@ checkLDAP(LDAP * persistent_ld, const char *userid, const char *password, const
 	char escaped_login[256];
 	LDAPMessage *res = NULL;
 	LDAPMessage *entry;
-	char *searchattr[] = {LDAP_NO_ATTRS, NULL};
+	char *searchattr[] =
+	{LDAP_NO_ATTRS, NULL};
 	char *userdn;
-	int rc;
 	LDAP *search_ld = persistent_ld;
 
 	if (!search_ld)
@@ -625,6 +680,8 @@ checkLDAP(LDAP * persistent_ld, const char *userid, const char *password, const
 		/* Everything is fine. This is expected when referrals
 		 * are disabled.
 		 */
+		if (debug)
+		    fprintf(stderr, "noreferrals && rc == LDAP_PARTIAL_RESULTS\n");
 	    } else {
 		fprintf(stderr, PROGRAM_NAME ": WARNING, LDAP search error '%s'\n", ldap_err2string(rc));
 #if defined(NETSCAPE_SSL)
@@ -639,6 +696,8 @@ checkLDAP(LDAP * persistent_ld, const char *userid, const char *password, const
 	}
 	entry = ldap_first_entry(search_ld, res);
 	if (!entry) {
+	    if (debug)
+		fprintf(stderr, "Ldap search returned nothing\n");
 	    ret = 1;
 	    goto search_done;
 	}
@@ -690,7 +749,7 @@ checkLDAP(LDAP * persistent_ld, const char *userid, const char *password, const
     return ret;
 }
 
-int 
+int
 readSecret(const char *filename)
 {
     char buf[BUFSIZ];
